from abc import ABC, abstractmethod
from typing import TYPE_CHECKING, Optional

if TYPE_CHECKING:
    from pystac import Item

from ..source import Source


class ItemBackend(ABC):
    def __repr__(self) -> str:
        return f"<{self.__class__.__name__}>"

    @abstractmethod
    def item_exists(self, source: Optional[Source], item: "Item") -> bool:
        ...

    @abstractmethod
    def register_item(self, source: Optional[Source], item: "Item", replace: bool):
        ...

    @abstractmethod
    def deregister_identifier(self, identifier: str) -> Optional[str]:
        ...


class PathBackend(ABC):
    def __repr__(self) -> str:
        return f"<{self.__class__.__name__}>"

    @abstractmethod
    def path_exists(self, source: Optional[Source], path: str) -> bool:
        ...

    @abstractmethod
    def register_path(self, source: Optional[Source], path: str, replace: bool):
        ...

    @abstractmethod
    def deregister_path(self, source: Optional[Source], path: str) -> Optional[str]:
        ...
